/*
Sample code for the magneto-mechanical filament diameter sensor.
Calibrate your individual sensor by measuring objects of known diameter and creating a custom lookup table!
*/

#define verbose 0 //0: No error or debugging messages via serial, 1: Error messages, 2: Error and debugging messages

#define NUMTEMPS 4  //length of lookup table
#define smooth 50  //exponential smoothing factor, higher is smoother
#define sensIn A1  //Output of the hall sensor

// ADC input, diameter output
static float lut3[NUMTEMPS][2] = {
   {532, 0},
   {675, 2.5},
   {870, 3},
   {992, 3.2}
};

float sensbuf =lut3[0][0]; //smoothed raw value



void setup() {
  Serial.begin(115200);
}

void loop() {
  int sensorValue = analogRead(sensIn);
  sensbuf += (sensorValue-sensbuf)/smooth; //smoothing
  Serial.print(sensbuf);
  Serial.print(" ");
  Serial.println(lookup(sensbuf, lut3),5);
  delay(10);        
}


float lookup(float inval, float lut[][2]){
   float out;
   byte i;
   for (i=1; i<NUMTEMPS; i++)
   {
      if (lut[i][0] > inval)
      {
         return lut[i-1][1] + (inval - lut[i-1][0]) * (lut[i][1] - lut[i-1][1]) / (lut[i][0] - lut[i-1][0]);
      }
   }
   if (i == NUMTEMPS){
     if (lut[0][1] > lut[NUMTEMPS][1])
     {
       #if verbose > 1
       Serial.println("WARNING: lookup out of range");
       #endif
       return lut[NUMTEMPS][1];
     }
     else
     {
       #if verbose > 1
       Serial.println("WARNING: lookup out of range");
       #endif
       return lut[0][1];
     }
   }
    #if verbose > 0
    Serial.println("ERROR: lookup failed");
    #endif
    return 0;
}
